 

import java.lang.*;
import java.lang.*;
import java.util.*;

/**
 * StringTokenizer qui utilise un string complet comme d?limiteur
 * et pas chaque caract?re s?par?ment.
 */

public class OtherStringTokenizer{
	private int currentPosition;
	private int newPosition;
	private int maxPosition;
	private String str;
	private String delimiters;
	private boolean retDelims;
	private boolean delimsChanged;
	
	
	/**
	 * Constructs a string tokenizer for the specified string. The String 
	 * in the <code>delim</code> argument is the delimiter for separating tokens. 
	 * <p>
	 * If the <code>returnDelims</code> flag is <code>true</code>, then 
	 * the delimiter characters are also returned as tokens. If the flag is 
	 * <code>false</code>, the delimiter string is skipped and only 
	 * serve as separator between tokens. 
	 *
	 * @param   str            a string to be parsed.
	 * @param   delim          the delimiter.
	 * @param   returnDelims   flag indicating whether to return the delimiter
	 *                         as tokens.
	 */
	public OtherStringTokenizer(String str, String delim, boolean returnDelims) {
		currentPosition = 0;
		newPosition = -1;
		delimsChanged = false;
		this.str = str;
		maxPosition = str.length();
		delimiters = delim;
		retDelims = returnDelims;
	}
	
	/**
	 * Constructs a string tokenizer for the specified string. The 
	 * string in the <code>delim</code> argument is the delimiter 
	 * for separating tokens. Delimiter string itself will not 
	 * be treated as a token.
	 *
	 * @param   str     a string to be parsed.
	 * @param   delim   the delimiter.
	 */
	public OtherStringTokenizer(String str, String delim) {
		this(str, delim, false);
	}
	
	/**
	 * Skips delimiters starting from the specified position. If retDelims
	 * is false, returns the index of the first non-delimiter character at or
	 * after startPos. If retDelims is true, startPos is returned.
	 */
	private int skipDelimiters(int startPos) {
		if (delimiters == null)
			throw new NullPointerException();
		
		int position = startPos;
		while (!retDelims && position < maxPosition) {
			if (!(str.substring(position)).startsWith(delimiters))
				break;
			position += delimiters.length();
		}
		if (position == maxPosition+1)
			position--;
		return position;
	}
	
	/**
	 * Skips ahead from startPos and returns the index of the next delimiter
	 * character encountered, or maxPosition if no such delimiter is found.
	 */
	private int scanToken(int startPos) {
		int position = startPos;
		while (position < maxPosition) {
			if ((str.substring(position)).startsWith(delimiters))
				break;
			position++;
		}
		if (retDelims && (startPos == position)) {
			if ((str.substring(position)).startsWith(delimiters))
				position += delimiters.length();
		}
		return position;
	}
	
	public boolean hasMoreTokens() {
		/*
		 * Temporary store this position and use it in the following
		 * nextToken() method only if the delimiters have'nt been changed in
		 * that nextToken() invocation.
		 */
		newPosition = skipDelimiters(currentPosition);
		return (newPosition < maxPosition);
	}
	
	/**
	 * Returns the next token from this string tokenizer.
	 *
	 * @return     the next token from this string tokenizer.
	 * @exception  NoSuchElementException  if there are no more tokens in this
	 *               tokenizer's string.
	 */
	public String nextToken() {
		/* 
		 * If next position already computed in hasMoreElements() and
		 * delimiters have changed between the computation and this invocation,
		 * then use the computed value.
		 */
		
		currentPosition = (newPosition >= 0 && !delimsChanged) ?  
			newPosition : skipDelimiters(currentPosition);
		
		/* Reset these anyway */
		delimsChanged = false;
		newPosition = -1;
		
		if (currentPosition >= maxPosition)
			throw new NoSuchElementException();
		int start = currentPosition;
		currentPosition = scanToken(currentPosition);
		return str.substring(start, currentPosition);
	}
	
	/**
	 * Returns the next token in this string tokenizer's string. First, 
	 * the set of characters considered to be delimiters by this 
	 * <tt>StringTokenizer</tt> object is changed to be the characters in 
	 * the string <tt>delim</tt>. Then the next token in the string
	 * after the current position is returned. The current position is 
	 * advanced beyond the recognized token.  The new delimiter set 
	 * remains the default after this call. 
	 *
	 * @param      delim   the new delimiters.
	 * @return     the next token, after switching to the new delimiter set.
	 * @exception  NoSuchElementException  if there are no more tokens in this
	 *               tokenizer's string.
	 */
	public String nextToken(String delim) {
		delimiters = delim;
		
		/* delimiter string specified, so set the appropriate flag. */
		delimsChanged = true;
		
		return nextToken();
	}
	
	/**
	 * Returns the same value as the <code>hasMoreTokens</code>
	 * method. It exists so that this class can implement the
	 * <code>Enumeration</code> interface. 
	 *
	 * @return  <code>true</code> if there are more tokens;
	 *          <code>false</code> otherwise.
	 * @see     java.util.Enumeration
	 * @see     java.util.StringTokenizer#hasMoreTokens()
	 */
	public boolean hasMoreElements() {
		return hasMoreTokens();
	}
	
	/**
	 * Returns the same value as the <code>nextToken</code> method,
	 * except that its declared return value is <code>Object</code> rather than
	 * <code>String</code>. It exists so that this class can implement the
	 * <code>Enumeration</code> interface. 
	 *
	 * @return     the next token in the string.
	 * @exception  NoSuchElementException  if there are no more tokens in this
	 *               tokenizer's string.
	 * @see        java.util.Enumeration
	 * @see        java.util.StringTokenizer#nextToken()
	 */
	public Object nextElement() {
		return nextToken();
	}
	
	/**
	 * Calculates the number of times that this tokenizer's 
	 * <code>nextToken</code> method can be called before it generates an 
	 * exception. The current position is not advanced.
	 *
	 * @return  the number of tokens remaining in the string using the current
	 *          delimiter set.
	 * @see     java.util.StringTokenizer#nextToken()
	 */
	public int countTokens() {
		int count = 0;
		int currpos = currentPosition;
		while (currpos < maxPosition) {
			currpos = skipDelimiters(currpos);
			if (currpos >= maxPosition)
				break;
			currpos = scanToken(currpos);
			count++;
		}
		return count;
	}
}

